package com.fzm.portal.controller.open;

import com.fzm.common.constant.SystemConstant;
import com.fzm.common.enums.CopyrightApplyState;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.open.entity.dataobject.CopyrightApiDO;
import com.fzm.common.open.entity.dataobject.OpenKeyDO;
import com.fzm.common.open.entity.openobject.BaseOO;
import com.fzm.common.open.entity.openobject.CopyrightAuthorOO;
import com.fzm.common.open.entity.openobject.CopyrightOO;
import com.fzm.common.open.entity.openobject.CopyrightOwnerOO;
import com.fzm.common.open.entity.queryobject.CopyrightStateQO;
import com.fzm.common.open.entity.viewobject.CopyrightStateVO;
import com.fzm.common.open.service.CopyrightApiService;
import com.fzm.common.open.service.OpenKeyService;
import com.fzm.common.utils.OpenSignUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author wt
 * @date 2022/4/24
 */
@RestController
@Api(tags = "Api版权服务")
@RequestMapping("/api/copyright")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class CopyrightApiController {
    private final OpenKeyService openKeyService;
    private final CopyrightApiService copyrightApiService;

    @PostMapping("/apply")
    @ApiOperation("/申请版权")
    public ResponseModel<Integer> apply(@Validated @RequestBody BaseOO<CopyrightOO> baseOO) {
        String appKey = baseOO.getAppKey();
        OpenKeyDO openKeyDO = openKeyService.getByAppKey(appKey);
        if (openKeyDO == null) {
            throw GlobalException.newException(ResultCode.app_key_wrong);
        }

        String bizId = baseOO.getBizId();
        if (StringUtils.isBlank(bizId)) {
            throw GlobalException.newException(ResultCode.biz_id_wrong);
        }

        String signature = baseOO.getSignature();
        if (StringUtils.isBlank(signature)) {
            throw GlobalException.newException(ResultCode.signature_wrong);
        }

        CopyrightOO copyrightOO = baseOO.getData();
        OpenSignUtils.verify(appKey, openKeyDO.getAppKeySecret(), bizId, copyrightOO, signature);

        CopyrightApiDO apiDO = copyrightApiService.getByAppKeyAndBizId(appKey, bizId);
        if (apiDO != null) {
            return ResponseModel.success(apiDO.getId());
        }

        Integer publishState = copyrightOO.getPublishState();
        if (publishState != 0 && publishState != 1) {
            throw GlobalException.newException(ResultCode.publish_state_wrong);
        }
        // 如果发表状态是已发表的话,需要传入首次发表日期和省市
        if (SystemConstant.BOOLEAN_DATA_TRUE.equals(publishState)) {
            Date firstPublishDate = copyrightOO.getFirstPublishDate();
            if (firstPublishDate == null) {
                throw GlobalException.newException(ResultCode.first_publish_date_wrong);
            }
            String firstPublishCity = copyrightOO.getFirstPublishCity();
            if (StringUtils.isBlank(firstPublishCity)) {
                throw GlobalException.newException(ResultCode.first_publish_city_wrong);
            }
            String firstPublishProvince = copyrightOO.getFirstPublishProvince();
            if (StringUtils.isBlank(firstPublishProvince)) {
                throw GlobalException.newException(ResultCode.first_publish_province_wrong);
            }
        }

        List<CopyrightAuthorOO> authors = copyrightOO.getAuthors();
        for (CopyrightAuthorOO author : authors) {
            String signType = author.getSignType();
            if (!"本名".equals(signType) && !"别名".equals(signType) && !"匿名".equals(signType)) {
                throw GlobalException.newException(ResultCode.sign_type_wrong);
            }
        }

        List<CopyrightOwnerOO> owners = copyrightOO.getOwners();
        for (CopyrightOwnerOO owner : owners) {
            Integer type = owner.getType();
            if (type != 0 && type != 1) {
                throw GlobalException.newException(ResultCode.owner_type_wrong);
            }
            Integer isEffective = owner.getIsEffective();
            if (isEffective != 0 && isEffective != 1) {
                throw GlobalException.newException(ResultCode.is_effective_wrong);
            }
            if (isEffective == 0) {
                Date endDate = owner.getEffectiveEndDate();
                if (endDate == null) {
                    throw GlobalException.newException(ResultCode.effective_end_date_wrong);
                }
            }
            Integer idType = owner.getIdType();
            if (idType < 0 || idType > 7) {
                throw GlobalException.newException(ResultCode.id_type_wrong);
            }
            if (idType == 0) {
                String positivePhoto = owner.getPositivePhoto();
                if (StringUtils.isBlank(positivePhoto)) {
                    throw GlobalException.newException(ResultCode.positive_photo_wrong);
                }
                String backPhoto = owner.getBackPhoto();
                if (StringUtils.isBlank(backPhoto)) {
                    throw GlobalException.newException(ResultCode.back_photo_wrong);
                }
                String personalPhoto = owner.getPersonalPhoto();
                if (StringUtils.isBlank(personalPhoto)) {
                    throw GlobalException.newException(ResultCode.personal_photo_wrong);
                }
            } else {
                String certificatesPhoto = owner.getCertificatesPhoto();
                if (StringUtils.isBlank(certificatesPhoto)) {
                    throw GlobalException.newException(ResultCode.certificates_photo_wrong);
                }
            }
        }

        Integer apply = copyrightApiService.apply(baseOO, openKeyDO);
        return ResponseModel.success(apply);
    }

    @GetMapping(value = "/state")
    @ApiOperation(value = "获取状态")
    public ResponseModel<CopyrightStateVO> getCopyrightState(@RequestParam(required = false) Integer id,
                                                             @RequestParam(required = false) String appKey,
                                                             @RequestParam(required = false) String bizId) {
        CopyrightApiDO apiDO = null;
        if (id != null) {
            apiDO = copyrightApiService.getById(id);
        }
        if (StringUtils.isNoneBlank(appKey, bizId)) {
            apiDO = copyrightApiService.getByAppKeyAndBizId(appKey, bizId);
        }
        if (apiDO == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "请核对后再重试");
        }

        CopyrightStateVO stateVO = switchStateVO(apiDO);

        return ResponseModel.success(stateVO);
    }

    @PostMapping(value = "/state")
    @ApiOperation(value = "获取状态")
    public ResponseModel<List<CopyrightStateVO>> getCopyrightStatePost(@RequestBody CopyrightStateQO stateQO) {
        String appKey = stateQO.getAppKey();
        OpenKeyDO openKeyDO = openKeyService.getByAppKey(appKey);
        if (openKeyDO == null) {
            throw GlobalException.newException(ResultCode.app_key_wrong);
        }
        List<String> bizIds = stateQO.getBizIds();
        if (CollectionUtils.isEmpty(bizIds) || bizIds.size() > 60) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "请核对后再重试");
        }
        List<CopyrightApiDO> doList = copyrightApiService.getByAppKeyAndBizIds(stateQO);
        if (CollectionUtils.isEmpty(doList)) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "请核对后再重试");
        }

        List<CopyrightStateVO> stateVOS = new ArrayList<>(doList.size());

        doList.forEach(o -> stateVOS.add(switchStateVO(o)));

        return ResponseModel.success(stateVOS);
    }

    private CopyrightStateVO switchStateVO(CopyrightApiDO apiDO) {
        CopyrightApplyState applyState = CopyrightApplyState.getByState(apiDO.getRegisterState());

        CopyrightStateVO stateVO = new CopyrightStateVO();
        stateVO.setBizId(apiDO.getBizId());
        stateVO.setState(applyState.getCode());
        stateVO.setMessage(applyState.getValue());
        if (applyState.equals(CopyrightApplyState.REJECTED) || applyState.equals(CopyrightApplyState.FAILED)) {
            stateVO.setMessage(apiDO.getRejectReason());
        }
        if (applyState.equals(CopyrightApplyState.SUCCEEDED)) {
            stateVO.setRegisterCode(apiDO.getRegisterCode());
            stateVO.setRegisterOwner(apiDO.getRegisterOwner());
            stateVO.setRegisterTime(apiDO.getRegisterTime());
        }
        return stateVO;
    }
}
